// file		:	critical.cxx
// purpose	:	critical section classes
// (c) Allan Kelly 1998-1999

// System Includes -------------------
#include <stdexcept>
#if defined(WIN32)
#include <windows.h>
#else	// assume we are POSIX threads
#include <pthread.h>	
#endif

// Project Includes ------------------
// <none>

// Local includes --------------------
#include "critical.h"

#if defined(NAME_SPACES)
using namespace Accu;
#endif

// create and initialize critical section
CriticalSection::CriticalSection()
{
#if defined(WIN32)
	_criticalSection = (void*) new CRITICAL_SECTION;
	InitializeCriticalSection( (CRITICAL_SECTION*) _criticalSection);
#else
	pthread_mutexattr_t attrib;
	attrib.mutexkind = PTHREAD_MUTEX_RECURSIVE_NP;
	_criticalSection = new pthread_mutex_t;
	int r = pthread_mutex_init(
						static_cast<pthread_mutex_t*> (_criticalSection), 
						&attrib);
	if (0 != r)
	{
		throw std::runtime_error("Failed to initialise mutex");
	}
#endif
}

// destroy critical section
CriticalSection::~CriticalSection()
{
#if defined(WIN32)
	DeleteCriticalSection( (CRITICAL_SECTION*) _criticalSection);
#else
	pthread_mutex_destroy(static_cast<pthread_mutex_t*>(_criticalSection));
#endif
	delete _criticalSection;
}

void CriticalSection::EnterCriticalSection()
{
#if defined(WIN32)
	::EnterCriticalSection( (CRITICAL_SECTION*) _criticalSection);
#else
	pthread_mutex_lock(static_cast<pthread_mutex_t*>(_criticalSection));
#endif
}

void CriticalSection::LeaveCriticalSection()
{
#if defined(WIN32)
	::LeaveCriticalSection( (CRITICAL_SECTION*) _criticalSection);
#else
	pthread_mutex_unlock(static_cast<pthread_mutex_t*>(_criticalSection));
#endif
}

CriticalToken::CriticalToken(CriticalSection &cs) : _criticalSection(cs)
{
	_criticalSection.EnterCriticalSection();
}

CriticalToken::~CriticalToken()
{
	_criticalSection.LeaveCriticalSection();
}

